/***********************************************************************
/*
/*    MyGainTool - An Audio Control Panel not dissimilar to the 
/*                 standard SunOS version...
/*
/*    Author:      David J. Singer <doc@trognet.demon.co.uk>
/* 
/*    Version:     V1.0, April 1997
/*
/*    Usage:       mygaintool [-s]
/*
/*                 Specifying the '-s' option indicates that volume 
/*                 level settings will be read from the resource file
/*                 $HOME/.mygaintoolrc (if present).
/*                 (A resource file will be created when the program is 
/*                 exited normally and '-s' was specified). 
/*
/*
/*  This program is free software; you can redistribute it and/or modify
/*  it under the terms of the GNU General Public License as published by
/*  the Free Software Foundation; either version 2 of the License, or
/*  (at your option) any later version.
/*
/*  This program is distributed in the hope that it will be useful, 
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/*  GNU General Public License for more details.
/*
/*  You should have received a copy of the GNU General Public License
/*  along with this program; if not, write to the Free Software
/*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
/*
/***********************************************************************/

#include <stdio.h>
#include <fcntl.h>
#include <X11/X.h>
#include <X11/Xlib.h>
#include <xview/xview.h>
#include <xview/frame.h>
#include <xview/panel.h>
#include <xview/icon_load.h>
#include <xview/svrimage.h>
#include <xview/defaults.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <strings.h>

#include "mygain.icon"
#include "mygain.mask.icon"

#define MASTER  0
#define PCM     1
#define CD      2
#define SYNTH   3
#define LINE    4
#define MIC     5


#define VERSION_INFO "V1.0 David J. Singer, Apr. 1997"


/* XView globals */ 
Frame           frame, about;
Panel           panel, panel;
Panel_item      vol_slider;
Server_image    icon_image, icon_mask;
Icon            icon;


/* Mixer globals */
static int      mixfd, channel=0;
static int      vol, treble, bass, value;
static int      master_vol, pcm_vol, cd_vol, synth_vol, line_vol, mic_vol;
static char	saveflag=0;
static char	rcfile[100];
FILE		*rcfd;



void get_mixer_levels( void )
{

   /* Read the mixer channel values from the soundcard */

   ioctl(mixfd, MIXER_READ(SOUND_MIXER_VOLUME), &vol);
   master_vol = vol & 0x000000ff;
   ioctl(mixfd, MIXER_READ(SOUND_MIXER_TREBLE), &vol);
   treble = vol & 0x000000ff;
   ioctl(mixfd, MIXER_READ(SOUND_MIXER_BASS), &vol);
   bass = vol & 0x000000ff;
   ioctl(mixfd, MIXER_READ(SOUND_MIXER_PCM), &vol);
   pcm_vol = vol & 0x000000ff;
   ioctl(mixfd, MIXER_READ(SOUND_MIXER_CD), &vol);
   cd_vol = vol & 0x000000ff;
   ioctl(mixfd, MIXER_READ(SOUND_MIXER_SYNTH), &vol);
   synth_vol = vol & 0x000000ff;
   ioctl(mixfd, MIXER_READ(SOUND_MIXER_LINE), &vol);
   line_vol = vol & 0x000000ff;
   ioctl(mixfd, MIXER_READ(SOUND_MIXER_MIC), &vol);
   mic_vol = vol & 0x000000ff;
}



void set_levels( void )
{

   /* Get levels at startup either from the resource file */
   /* if '-s' specified, or otherwise from the soundcard  */ 

   if (!saveflag || ((rcfd=fopen(rcfile, "rt"))==NULL) ) 
      /* No resource file, so get values from soundcard. */
      /* We're just interested in getting the left value */
      /* as we set both left and right to the same value */
      get_mixer_levels();
   else {
      /* Get values from resource file */
      fscanf(rcfd, "%d %d %d %d %d %d %d %d\n", &master_vol, &treble, &bass, \
             &pcm_vol, &cd_vol, &synth_vol, &line_vol, &mic_vol);
      fclose(rcfd);
   }
      
   /* Now write level settings to soundcard */

   /* master volume */
   vol = (master_vol & 0xff) | ((master_vol & 0xff) << 8);
   ioctl(mixfd, MIXER_WRITE(SOUND_MIXER_VOLUME), &vol);

   /* treble value */
   vol = (treble & 0xff) | ((treble & 0xff) << 8);
   ioctl(mixfd, MIXER_WRITE(SOUND_MIXER_TREBLE), &vol);

   /* bass value */
   vol = (bass & 0xff) | ((bass & 0xff) << 8);
   ioctl(mixfd, MIXER_WRITE(SOUND_MIXER_BASS), &vol);

   /* PCM value */
   vol = (pcm_vol & 0xff) | ((pcm_vol & 0xff) << 8);
   ioctl(mixfd, MIXER_WRITE(SOUND_MIXER_PCM), &vol);

   /* CD value */
   vol = (cd_vol & 0xff) | ((cd_vol & 0xff) << 8);
   ioctl(mixfd, MIXER_WRITE(SOUND_MIXER_CD), &vol);

   /* SYNTH value */
   vol = (synth_vol & 0xff) | ((synth_vol & 0xff) << 8);
   ioctl(mixfd, MIXER_WRITE(SOUND_MIXER_SYNTH), &vol);

   /* LINE value */
   vol = (line_vol & 0xff) | ((line_vol & 0xff) << 8);
   ioctl(mixfd, MIXER_WRITE(SOUND_MIXER_LINE), &vol);

   /* MIC value */
   vol = (mic_vol & 0xff) | ((mic_vol & 0xff) << 8);
   ioctl(mixfd, MIXER_WRITE(SOUND_MIXER_MIC), &vol);

   /* Clear all recording sources (driver defaults to mic) */
   ioctl(mixfd, SOUND_MIXER_WRITE_RECSRC, 0);

}



void save_settings( void )
{

   /* Write level settings to resource file */

   if ((rcfd=fopen(rcfile, "wt"))==NULL) 
      fprintf(stderr, "Cannot save settings to %s\n", rcfile);
   else {
      /* Get current levels from soundcard */
      get_mixer_levels();
      /* And save to the resource file */
      fprintf(rcfd, "%d %d %d %d %d %d %d %d\n", master_vol, treble, bass, \
              pcm_vol, cd_vol, synth_vol, line_vol, mic_vol);
      fprintf(rcfd, "MyGainTool: %s\nDO NOT EDIT THIS FILE!\n", VERSION_INFO);      
      chmod(rcfile, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
      fclose(rcfd);
   }

}


void treble_proc( item, value, event )
Panel_item item;
int value;
Event event;
{

   /* Write value to left and right of mixer treble channel */

   vol = (value & 0xff) | ((value & 0xff) << 8);
   ioctl(mixfd, MIXER_WRITE(SOUND_MIXER_TREBLE), &vol);

}



void bass_proc( item, value, event )
Panel_item item;
int value;
Event event;
{

   /* Write value to left and right of mixer bass channel */

   vol = (value & 0xff) | ((value & 0xff) << 8);
   ioctl(mixfd, MIXER_WRITE(SOUND_MIXER_BASS), &vol);

}



void select_chan_proc( item, value, event )
Panel_item item;
int value;
Event event;
{

   /* Set volume slider to selected channel's current value */

   channel = value;
   switch (channel) {
      case MASTER : vol = master_vol; 
                    break;;

      case PCM    : vol = pcm_vol;
                    break;;

      case CD     : vol = cd_vol;     
                    break;;

      case SYNTH  : vol = synth_vol;  
                    break;;

      case LINE   : vol = line_vol;  
                    break;;

      case MIC    : vol = mic_vol;    
                    break;;
 
   }     

   xv_set(vol_slider, PANEL_VALUE, vol, NULL);

}



void level_proc( item, value, event )
Panel_item item;
int value;
Event event;
{
   int mixer_channel;

   /* Write value to left and right of current channel */

   switch (channel) {
      case MASTER : master_vol = value; 
                    mixer_channel = SOUND_MIXER_VOLUME;
                    break;;

      case PCM    : pcm_vol = value;    
                    mixer_channel = SOUND_MIXER_PCM;                     
                    break;;

      case CD     : cd_vol = value;
                    mixer_channel = SOUND_MIXER_CD;
                    break;;

      case SYNTH  : synth_vol = value;
                    mixer_channel = SOUND_MIXER_SYNTH;
                    break;;

      case LINE   : line_vol = value;
                    mixer_channel = SOUND_MIXER_LINE;
                    break;;

      case MIC    : mic_vol = value;
                    mixer_channel = SOUND_MIXER_MIC;
                    break;;

   }

   vol = (value & 0xff) | ((value & 0xff)<<8);
   ioctl(mixfd, MIXER_WRITE(mixer_channel), &vol);
   
}


void draw_gui()
{


    frame = (Frame) xv_create (XV_NULL, FRAME,
        FRAME_LABEL,                "Audio Control Panel",
        FRAME_SHOW_FOOTER,          FALSE,
        FRAME_SHOW_RESIZE_CORNER,   FALSE,
        XV_WIDTH,                   290,
        XV_HEIGHT,                  94,
        NULL );

    /* Create Icon image */
    icon_image = (Server_image) xv_create (XV_NULL, SERVER_IMAGE,   
        XV_WIDTH,               icon_width,
        XV_HEIGHT,              icon_height,
        SERVER_IMAGE_X_BITS,    icon_bits,
        NULL );

    /* Create Icon image mask */
    icon_mask = (Server_image) xv_create (XV_NULL, SERVER_IMAGE,   
        XV_WIDTH,               icon_mask_width,
        XV_HEIGHT,              icon_mask_height,
        SERVER_IMAGE_X_BITS,    icon_mask_bits,
        NULL );

    /* Define Icon */
    icon = (Icon) xv_create (frame, ICON,
        ICON_IMAGE,        icon_image,
        ICON_MASK_IMAGE,   icon_mask,
        ICON_TRANSPARENT,  TRUE,  
        NULL );

    /* Allocate Icon to Frame */
    xv_set (frame, FRAME_ICON, icon, NULL);


    /* Define a panel */
    panel = (Panel) xv_create (frame, PANEL, 
        XV_X,            0,
        XV_Y,            0, 
        XV_WIDTH,        WIN_EXTEND_TO_EDGE,
        XV_HEIGHT,       WIN_EXTEND_TO_EDGE,
        WIN_BORDER,      TRUE,
        NULL );

    /* Create the 'Play Volume' slider */
    vol_slider = xv_create(panel, PANEL_SLIDER,
        XV_X,                    20,
        XV_Y,                    6,
        PANEL_SLIDER_WIDTH,      135,
        PANEL_TICKS,             0,
        PANEL_LABEL_STRING,      "Volume:",
        PANEL_DIRECTION,         PANEL_HORIZONTAL,
        PANEL_SLIDER_END_BOXES,  FALSE,
        PANEL_SHOW_RANGE,        FALSE,
        PANEL_SHOW_VALUE,        TRUE,
        PANEL_MIN_VALUE,         0,
        PANEL_MAX_VALUE,         100,
        PANEL_VALUE,             master_vol,
        PANEL_NOTIFY_LEVEL,      PANEL_ALL,
        PANEL_NOTIFY_PROC,       level_proc,
        NULL);

    /* Create the Treble slider */
    xv_create(panel, PANEL_SLIDER,
        XV_X,                    27,
        XV_Y,                    24,
        PANEL_SLIDER_WIDTH,      135,
        PANEL_TICKS,             0,
        PANEL_LABEL_STRING,      "Treble:",
        PANEL_DIRECTION,         PANEL_HORIZONTAL,
        PANEL_SLIDER_END_BOXES,  FALSE,
	PANEL_SHOW_RANGE,        FALSE,
	PANEL_SHOW_VALUE,        TRUE,
	PANEL_MIN_VALUE,         0,
	PANEL_MAX_VALUE,         100,
	PANEL_VALUE,             treble,
        PANEL_NOTIFY_LEVEL,      PANEL_ALL,
	PANEL_NOTIFY_PROC,       treble_proc,
	NULL);

    /* Create Bass slider */
    xv_create(panel, PANEL_SLIDER,
	XV_X,                    39,
	XV_Y,                    42,
	PANEL_SLIDER_WIDTH,      135,
	PANEL_TICKS,             0,
	PANEL_LABEL_STRING,      "Bass:",
	PANEL_DIRECTION,         PANEL_HORIZONTAL,
	PANEL_SLIDER_END_BOXES,  FALSE,
	PANEL_SHOW_RANGE,        FALSE,
	PANEL_SHOW_VALUE,        TRUE,
	PANEL_MIN_VALUE,         0,
	PANEL_MAX_VALUE,         100,
	PANEL_VALUE,             bass,
        PANEL_NOTIFY_LEVEL,      PANEL_ALL,
	PANEL_NOTIFY_PROC,       bass_proc,
	NULL);

    /* Create Channel select switch */
    xv_create(panel, PANEL_CHOICE,
	XV_X,                    12,
	XV_Y,                    65,
	PANEL_CHOICE_NROWS,      1,
	PANEL_LAYOUT,            PANEL_HORIZONTAL,
	PANEL_CHOOSE_NONE,       FALSE,
	PANEL_NOTIFY_PROC,       select_chan_proc,
	PANEL_CHOICE_STRINGS,    "Master", "PCM", "CD", "Synth", 
                                 "Line", "Mic", NULL,
	NULL);
   
	
}



main(argc, argv)
int argc;
char *argv[];
{

   /* Initialse mixer device */
   if ((mixfd=open("/dev/mixer",O_RDWR, 0)) < 0) {
      fprintf(stderr, "mygaintool: cannot open /dev/mixer\n");
      exit(1);
   }

   /* Initialise XView system */
   xv_init (XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);

   /* Are we loading/saving settings to resource file? */
   if (argc>1 && argv[1][1]=='s') {
     saveflag=1;
     /* Get resource file name */
     strcpy(rcfile, getenv("HOME"));
     strcat(rcfile, "/.mygaintoolrc");
   }

   /* Get default channel levels */
   set_levels();

   /* Set up Graphical User Interface */
   draw_gui();

   /* Main event processing */
   xv_main_loop (frame);

   /* If we're saving the settings, do so now */
   if (saveflag)
     save_settings();

   exit(0);

}


